# coding: utf-8


"""
IGNORE:
    Nutanix Flow Management APIs

    Manage Network Security Policy configuration of Nutanix clusters.  # noqa: E501

    OpenAPI spec version: 4.2.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
from __future__ import absolute_import

import io
import json
import logging
import re
import ssl

import certifi
# python 2 and python 3 compatibility library
import six
from six.moves.urllib.parse import urlencode

try:
    import urllib3
    from urllib3.contrib.socks import SOCKSProxyManager
except ImportError:
    raise ImportError('Swagger python client requires urllib3.')

from .custom_retry import CustomPoolManager

logger = logging.getLogger(__name__)


class RESTResponse(io.IOBase):

    def __init__(self, resp):
        self.urllib3_response = resp
        self.status = resp.status
        self.reason = resp.reason
        self.data = None

    def getheaders(self):
        """Returns a dictionary of the response headers."""
        return self.urllib3_response.headers

    def getheader(self, name, default=None):
        """Returns a given response header."""
        return self.urllib3_response.headers.get(name, default)


class RESTClientObject(object):

    # Allowed methods to be used for retry
    RETRY_ALLOWED_METHODS = frozenset(["HEAD", "GET", "PUT", "DELETE", "OPTIONS", "TRACE", "POST"])

    def __init__(self, configuration, pools_size=4, maxsize=None, max_retry_attempts=5, backoff_factor=3, max_redirects=1):
        # urllib3.PoolManager will pass all kw parameters to connectionpool
        # https://github.com/shazow/urllib3/blob/f9409436f83aeb79fbaf090181cd81b784f1b8ce/urllib3/poolmanager.py#L75  # noqa: E501
        # https://github.com/shazow/urllib3/blob/f9409436f83aeb79fbaf090181cd81b784f1b8ce/urllib3/connectionpool.py#L680  # noqa: E501
        # maxsize is the number of requests to host that are allowed in parallel  # noqa: E501
        # Custom SSL certificates and client certificates: http://urllib3.readthedocs.io/en/latest/advanced-usage.html  # noqa: E501

        self.__configuration = configuration

        # cert_reqs
        if configuration.verify_ssl:
            cert_reqs = ssl.CERT_REQUIRED
        else:
            cert_reqs = ssl.CERT_NONE

        # ca_certs
        if configuration.root_ca_certificate_file:
            ca_certs = configuration.root_ca_certificate_file
        else:
            # if not set certificate file, use Mozilla's root certificates.
            ca_certs = certifi.where()

        addition_pool_args = {}
        if configuration.assert_hostname is not None:
            addition_pool_args['assert_hostname'] = configuration.assert_hostname  # noqa: E501

        if maxsize is None:
            if configuration.connection_pool_maxsize is not None:
                maxsize = configuration.connection_pool_maxsize
            else:
                maxsize = 4

        # retry mechanism
        retries = urllib3.util.Retry(total=None, status=max_retry_attempts, read=0, connect=0, other=0, allowed_methods= self.RETRY_ALLOWED_METHODS, status_forcelist=[408, 429, 503, 504], backoff_factor=backoff_factor, raise_on_status=False, redirect=False)

        # https pool manager
        if configuration.proxy_scheme and configuration.proxy_host and configuration.proxy_port:
            proxy_url = ''.join([
                configuration.proxy_scheme,
                '://',
                configuration.proxy_host,
                ':',
                str(configuration.proxy_port)
            ])
            proxy_headers = (
                urllib3.util.make_headers(
                proxy_basic_auth=configuration.proxy_username + ':' + configuration.proxy_password) if
                (configuration.proxy_username and configuration.proxy_password) else None
            )
            if configuration.proxy_scheme == "socks5":
                self.pool_manager = SOCKSProxyManager(
                    num_pools=pools_size,
                    maxsize=maxsize,
                    cert_reqs=cert_reqs,
                    ca_certs=ca_certs,
                    cert_file=configuration.client_certificate_file,
                    key_file=configuration.client_key_file,
                    proxy_url=proxy_url,
                    username=configuration.proxy_username,
                    password=configuration.proxy_password,
                    retries = retries,
                    **addition_pool_args
                )
            else:
                self.pool_manager = urllib3.ProxyManager(
                    num_pools=pools_size,
                    maxsize=maxsize,
                    cert_reqs=cert_reqs,
                    ca_certs=ca_certs,
                    cert_file=configuration.client_certificate_file,
                    key_file=configuration.client_key_file,
                    proxy_url=proxy_url,
                    proxy_headers=proxy_headers,
                    retries = retries,
                    **addition_pool_args
                )
        else:
            self.pool_manager = CustomPoolManager(
                pools_size,
                maxsize,
                cert_reqs,
                ca_certs,
                configuration.client_certificate_file,
                configuration.client_key_file,
                retries,
                **addition_pool_args
            )

    def request(self, method, url, query_params=None, headers=None,
                body=None, post_params=None, _preload_content=True,
                _request_timeout=None):
        """Perform requests.

        :param method: http request method
        :param url: http request url
        :param query_params: query parameters in the url
        :param headers: http request headers
        :param body: request json body, for `application/json`
        :param post_params: request post parameters,
                            `application/x-www-form-urlencoded`
                            and `multipart/form-data`
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        """
        method = method.upper()
        assert method in ['GET', 'HEAD', 'DELETE', 'POST', 'PUT',
                          'PATCH', 'OPTIONS']

        if post_params and body:
            msg = "body parameter cannot be used with post_params parameter."
            logger.error(msg)
            raise ValueError(msg)

        post_params = post_params or {}
        headers = headers or {}

        timeout = None
        if _request_timeout:
            if isinstance(_request_timeout, (int, float) if six.PY3 else (int, long, float)):  # noqa: E501,F821
                timeout = urllib3.Timeout(total=_request_timeout)
            elif (isinstance(_request_timeout, list) and len(_request_timeout) == 2):
                timeout = urllib3.Timeout(connect=_request_timeout[0], read=_request_timeout[1])

        if 'Content-Type' not in headers:
            headers['Content-Type'] = 'application/json'

        _preload_content = False

        # Print log statements
        logger.info("%s %s", method, url + '?' + urlencode(query_params) if query_params else url)
        # Print debug logs, if enabled
        if self.__configuration.debug:
            logger.debug("Request Headers:")
            for key, value in six.iteritems(headers):
                logger.debug("%s: %s", key, value)

        try:
            # For `POST`, `PUT`, `PATCH`, `OPTIONS`, `DELETE`
            if method in ['POST', 'PUT', 'PATCH', 'OPTIONS', 'DELETE']:
                if query_params:
                    url += '?' + urlencode(query_params)
                if re.search('json', headers['Content-Type'], re.IGNORECASE):
                    request_body = None
                    if body is not None:
                        request_body = json.dumps(body)

                    # Print request body, if debug configuration is enabled
                    if self.__configuration.debug:
                        logger.debug("Request Body: %s", request_body)

                    r = self.pool_manager.request(
                        method, url,
                        body=request_body,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                elif headers['Content-Type'] == 'application/x-www-form-urlencoded':  # noqa: E501
                    r = self.pool_manager.request(
                        method, url,
                        fields=post_params,
                        encode_multipart=False,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                elif headers['Content-Type'] == 'multipart/form-data':
                    # must del headers['Content-Type'], or the correct
                    # Content-Type which generated by urllib3 will be
                    # overwritten.
                    content_type_header = headers['Content-Type']
                    del headers['Content-Type']
                    r = self.pool_manager.request(
                        method, url,
                        fields=post_params,
                        encode_multipart=True,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                    headers['Content-Type'] = content_type_header
                # Pass a `string` parameter directly in the body to support
                # other content types than Json when `body` argument is
                # provided in serialized form
                elif isinstance(body, str) or headers['Content-Type'] == 'application/octet-stream':
                    request_body = body
                    preload = _preload_content
                    if headers['Content-Type'] == 'application/octet-stream':
                        preload = False

                    r = self.pool_manager.request(
                        method, url,
                        body=request_body,
                        preload_content=preload,
                        timeout=timeout,
                        headers=headers)
                else:
                    # Cannot generate the request from given parameters
                    msg = """Cannot prepare a request message for provided
                             arguments. Please check that your arguments match
                             declared content type."""
                    raise ApiException(status=0, reason=msg)
            # For `GET`, `HEAD`
            else:
                r = self.pool_manager.request(method, url,
                                              fields=query_params,
                                              preload_content=_preload_content,
                                              timeout=timeout,
                                              headers=headers)
        except urllib3.exceptions.SSLError as e:
            msg = "{0}\n{1}".format(type(e).__name__, str(e))
            raise ApiException(status=0, reason=msg)
        except Exception as e:
            logger.error(str(e))
            raise

        # HANDLE REDIRECT MANUALLY
        if r.status == 302:
            logger.info("Received a redirect [302] request")
            # Handle redirection manually
            location = r.headers.get('Location')
            logger.info("Redirecting request from %s to %s ", url, location)
            headers = headers.copy()
            if location:
                # Handle custom header case
                if 'X-Redirect-Token' in r.headers:
                    logger.debug("X-Redirect-Token found")
                    custom_header_value = r.headers['X-Redirect-Token']
                    headers['Cookie'] = custom_header_value
                else:
                    if 'Authorization' in headers:
                        del headers['Authorization']
                    if 'Cookie' in headers:
                        del headers['Cookie']

                # Make a new request to the URL specified in the Location header
                redirect_response = self.pool_manager.request(method, location,
                                                              fields=query_params,
                                                              preload_content=_preload_content,
                                                              timeout=timeout,
                                                              headers=headers)
                r = redirect_response
            else:
                logger.error("Redirect location not found in 302 response headers.")

        r = RESTResponse(r)

        # Print log statements
        logger.info("Response Status: %s %s", r.status, r.reason)
        # Print debug logs, if enabled
        if self.__configuration.debug:
            logger.debug("Response Headers:")
            for key, value in six.iteritems(r.getheaders()):
                logger.debug("%s: %s", key, value)

            if r.getheader("Content-Type", "application/json") not in ['application/octet-stream', 'application/pdf', 'application/zip']:
                logger.debug("Response Body: %s", r.urllib3_response.data)

        if not 200 <= r.status <= 299 and not 401 == r.status:
            raise ApiException(http_resp=r)

        return r

    def GET(self, url, headers=None, query_params=None, _preload_content=True,
            _request_timeout=None):
        return self.request("GET", url,
                            headers=headers,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            query_params=query_params)

    def HEAD(self, url, headers=None, query_params=None, _preload_content=True,
             _request_timeout=None):
        return self.request("HEAD", url,
                            headers=headers,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            query_params=query_params)

    def OPTIONS(self, url, headers=None, query_params=None, post_params=None,
                body=None, _preload_content=True, _request_timeout=None):
        return self.request("OPTIONS", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def DELETE(self, url, headers=None, query_params=None, body=None,
               _preload_content=True, _request_timeout=None):
        return self.request("DELETE", url,
                            headers=headers,
                            query_params=query_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def POST(self, url, headers=None, query_params=None, post_params=None,
             body=None, _preload_content=True, _request_timeout=None):
        return self.request("POST", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def PUT(self, url, headers=None, query_params=None, post_params=None,
            body=None, _preload_content=True, _request_timeout=None):
        return self.request("PUT", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def PATCH(self, url, headers=None, query_params=None, post_params=None,
              body=None, _preload_content=True, _request_timeout=None):
        return self.request("PATCH", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)


class ApiException(Exception):

    def __init__(self, status=None, reason=None, http_resp=None):
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = (http_resp.urllib3_response.data.decode('utf8') if hasattr(http_resp, 'urllib3_response')
                        else http_resp.data.decode('utf8'))
            self.headers = http_resp.getheaders()
        else:
            self.status = status
            self.reason = reason
            self.body = None
            self.headers = None

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "({0})\n"\
                        "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(
                self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        logger.error(error_message)
        return error_message


# coding: utf-8

# flake8: noqa

"""
IGNORE:
    Nutanix Flow Management APIs

    Manage Network Security Policy configuration of Nutanix clusters.  # noqa: E501

    OpenAPI spec version: 4.2.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
from __future__ import absolute_import

# import models into model package
from ntnx_microseg_py_client.models.OneOfcommon.v1.config.KVPairvalue import KVPairvalue
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.CreateAddressGroupApiResponsedata import CreateAddressGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.CreateDirectoryServerConfigApiResponsedata import CreateDirectoryServerConfigApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.CreateDsCategoryMappingApiResponsedata import CreateDsCategoryMappingApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.CreateEntityGroupApiResponsedata import CreateEntityGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.CreateNetworkSecurityPolicyApiResponsedata import CreateNetworkSecurityPolicyApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.CreateNetworkSecurityPolicyExportApiResponsedata import CreateNetworkSecurityPolicyExportApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.CreateNetworkSecurityPolicyImportApiResponsedata import CreateNetworkSecurityPolicyImportApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.CreateServiceGroupApiResponsedata import CreateServiceGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.DeleteAddressGroupApiResponsedata import DeleteAddressGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.DeleteDirectoryServerConfigApiResponsedata import DeleteDirectoryServerConfigApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.DeleteDsCategoryMappingApiResponsedata import DeleteDsCategoryMappingApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.DeleteEntityGroupApiResponsedata import DeleteEntityGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.DeleteNetworkSecurityPolicyApiResponsedata import DeleteNetworkSecurityPolicyApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.DeleteServiceGroupApiResponsedata import DeleteServiceGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.GetAddressGroupApiResponsedata import GetAddressGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.GetDirectoryServerConfigApiResponsedata import GetDirectoryServerConfigApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.GetDsCategoryMappingApiResponsedata import GetDsCategoryMappingApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.GetEntityGroupApiResponsedata import GetEntityGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.GetNetworkSecurityPolicyApiResponsedata import GetNetworkSecurityPolicyApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.GetServiceGroupApiResponsedata import GetServiceGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.ListAddressGroupsApiResponsedata import ListAddressGroupsApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.ListDirectoryServerConfigsApiResponsedata import ListDirectoryServerConfigsApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.ListDsCategoryMappingsApiResponsedata import ListDsCategoryMappingsApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.ListEntityGroupsApiResponsedata import ListEntityGroupsApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.ListNetworkSecurityPoliciesApiResponsedata import ListNetworkSecurityPoliciesApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.ListNetworkSecurityPolicyRulesApiResponsedata import ListNetworkSecurityPolicyRulesApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.ListServiceGroupsApiResponsedata import ListServiceGroupsApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.MultiEnvIsolationRuleSpecspec import MultiEnvIsolationRuleSpecspec
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.NetworkSecurityPolicyRulespec import NetworkSecurityPolicyRulespec
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.UpdateAddressGroupApiResponsedata import UpdateAddressGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.UpdateDirectoryServerConfigApiResponsedata import UpdateDirectoryServerConfigApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.UpdateDsCategoryMappingApiResponsedata import UpdateDsCategoryMappingApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.UpdateEntityGroupApiResponsedata import UpdateEntityGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.UpdateNetworkSecurityPolicyApiResponsedata import UpdateNetworkSecurityPolicyApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.UpdateServiceGroupApiResponsedata import UpdateServiceGroupApiResponsedata
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.error.ErrorResponseerror import ErrorResponseerror
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.operation.UpgradeJobspec import UpgradeJobspec
from ntnx_microseg_py_client.models.common.v1.config.FQDN import FQDN
from ntnx_microseg_py_client.models.common.v1.config.Flag import Flag
from ntnx_microseg_py_client.models.common.v1.config.IPAddressOrFQDN import IPAddressOrFQDN
from ntnx_microseg_py_client.models.common.v1.config.IPv4Address import IPv4Address
from ntnx_microseg_py_client.models.common.v1.config.IPv6Address import IPv6Address
from ntnx_microseg_py_client.models.common.v1.config.KVPair import KVPair
from ntnx_microseg_py_client.models.common.v1.config.MapOfStringWrapper import MapOfStringWrapper
from ntnx_microseg_py_client.models.common.v1.config.Message import Message
from ntnx_microseg_py_client.models.common.v1.config.MessageSeverity import MessageSeverity
from ntnx_microseg_py_client.models.common.v1.config.TenantAwareModel import TenantAwareModel
from ntnx_microseg_py_client.models.common.v1.response.ApiLink import ApiLink
from ntnx_microseg_py_client.models.common.v1.response.ApiResponseMetadata import ApiResponseMetadata
from ntnx_microseg_py_client.models.common.v1.response.ExternalizableAbstractModel import ExternalizableAbstractModel
from ntnx_microseg_py_client.models.microseg.v4.config.AdInfo import AdInfo
from ntnx_microseg_py_client.models.microseg.v4.config.AdStatus import AdStatus
from ntnx_microseg_py_client.models.microseg.v4.config.AddressGroup import AddressGroup
from ntnx_microseg_py_client.models.microseg.v4.config.Addresses import Addresses
from ntnx_microseg_py_client.models.microseg.v4.config.AllToAllIsolationGroup import AllToAllIsolationGroup
from ntnx_microseg_py_client.models.microseg.v4.config.AllowType import AllowType
from ntnx_microseg_py_client.models.microseg.v4.config.AllowedConfig import AllowedConfig
from ntnx_microseg_py_client.models.microseg.v4.config.AllowedEntity import AllowedEntity
from ntnx_microseg_py_client.models.microseg.v4.config.AllowedSelectBy import AllowedSelectBy
from ntnx_microseg_py_client.models.microseg.v4.config.AllowedType import AllowedType
from ntnx_microseg_py_client.models.microseg.v4.config.ApplicationRuleSpec import ApplicationRuleSpec
from ntnx_microseg_py_client.models.microseg.v4.config.CategoryAssociatedEntityType import CategoryAssociatedEntityType
from ntnx_microseg_py_client.models.microseg.v4.config.CategoryMapping import CategoryMapping
from ntnx_microseg_py_client.models.microseg.v4.config.CreateAddressGroupApiResponse import CreateAddressGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.CreateDirectoryServerConfigApiResponse import CreateDirectoryServerConfigApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.CreateDsCategoryMappingApiResponse import CreateDsCategoryMappingApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.CreateEntityGroupApiResponse import CreateEntityGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.CreateNetworkSecurityPolicyApiResponse import CreateNetworkSecurityPolicyApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.CreateNetworkSecurityPolicyExportApiResponse import CreateNetworkSecurityPolicyExportApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.CreateNetworkSecurityPolicyImportApiResponse import CreateNetworkSecurityPolicyImportApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.CreateServiceGroupApiResponse import CreateServiceGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.DeleteAddressGroupApiResponse import DeleteAddressGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.DeleteDirectoryServerConfigApiResponse import DeleteDirectoryServerConfigApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.DeleteDsCategoryMappingApiResponse import DeleteDsCategoryMappingApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.DeleteEntityGroupApiResponse import DeleteEntityGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.DeleteNetworkSecurityPolicyApiResponse import DeleteNetworkSecurityPolicyApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.DeleteServiceGroupApiResponse import DeleteServiceGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.DirectoryServerConfig import DirectoryServerConfig
from ntnx_microseg_py_client.models.microseg.v4.config.EntityGroup import EntityGroup
from ntnx_microseg_py_client.models.microseg.v4.config.ExceptConfig import ExceptConfig
from ntnx_microseg_py_client.models.microseg.v4.config.ExceptEntity import ExceptEntity
from ntnx_microseg_py_client.models.microseg.v4.config.ExceptSelectBy import ExceptSelectBy
from ntnx_microseg_py_client.models.microseg.v4.config.ExceptType import ExceptType
from ntnx_microseg_py_client.models.microseg.v4.config.GetAddressGroupApiResponse import GetAddressGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.GetDirectoryServerConfigApiResponse import GetDirectoryServerConfigApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.GetDsCategoryMappingApiResponse import GetDsCategoryMappingApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.GetEntityGroupApiResponse import GetEntityGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.GetNetworkSecurityPolicyApiResponse import GetNetworkSecurityPolicyApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.GetServiceGroupApiResponse import GetServiceGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.IPv4Range import IPv4Range
from ntnx_microseg_py_client.models.microseg.v4.config.IPv6Range import IPv6Range
from ntnx_microseg_py_client.models.microseg.v4.config.IcmpTypeCodeSpec import IcmpTypeCodeSpec
from ntnx_microseg_py_client.models.microseg.v4.config.IcmpV6TypeCodeSpec import IcmpV6TypeCodeSpec
from ntnx_microseg_py_client.models.microseg.v4.config.IntraEntityGroupRuleAction import IntraEntityGroupRuleAction
from ntnx_microseg_py_client.models.microseg.v4.config.IntraEntityGroupRuleSpec import IntraEntityGroupRuleSpec
from ntnx_microseg_py_client.models.microseg.v4.config.IpRange import IpRange
from ntnx_microseg_py_client.models.microseg.v4.config.IsolationGroup import IsolationGroup
from ntnx_microseg_py_client.models.microseg.v4.config.ListAddressGroupsApiResponse import ListAddressGroupsApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.ListDirectoryServerConfigsApiResponse import ListDirectoryServerConfigsApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.ListDsCategoryMappingsApiResponse import ListDsCategoryMappingsApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.ListEntityGroupsApiResponse import ListEntityGroupsApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.ListNetworkSecurityPoliciesApiResponse import ListNetworkSecurityPoliciesApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.ListNetworkSecurityPolicyRulesApiResponse import ListNetworkSecurityPolicyRulesApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.ListServiceGroupsApiResponse import ListServiceGroupsApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.MatchEntity import MatchEntity
from ntnx_microseg_py_client.models.microseg.v4.config.MatchField import MatchField
from ntnx_microseg_py_client.models.microseg.v4.config.MatchType import MatchType
from ntnx_microseg_py_client.models.microseg.v4.config.MatchingCriteria import MatchingCriteria
from ntnx_microseg_py_client.models.microseg.v4.config.MultiEnvIsolationRuleSpec import MultiEnvIsolationRuleSpec
from ntnx_microseg_py_client.models.microseg.v4.config.NetworkSecurityPolicy import NetworkSecurityPolicy
from ntnx_microseg_py_client.models.microseg.v4.config.NetworkSecurityPolicyExportSpec import NetworkSecurityPolicyExportSpec
from ntnx_microseg_py_client.models.microseg.v4.config.NetworkSecurityPolicyRule import NetworkSecurityPolicyRule
from ntnx_microseg_py_client.models.microseg.v4.config.RuleType import RuleType
from ntnx_microseg_py_client.models.microseg.v4.config.SecurityPolicyScope import SecurityPolicyScope
from ntnx_microseg_py_client.models.microseg.v4.config.SecurityPolicyState import SecurityPolicyState
from ntnx_microseg_py_client.models.microseg.v4.config.SecurityPolicyType import SecurityPolicyType as ConfigSecurityPolicyType
from ntnx_microseg_py_client.models.microseg.v4.config.ServiceGroup import ServiceGroup
from ntnx_microseg_py_client.models.microseg.v4.config.SyncPolicy import SyncPolicy
from ntnx_microseg_py_client.models.microseg.v4.config.TcpPortRangeSpec import TcpPortRangeSpec
from ntnx_microseg_py_client.models.microseg.v4.config.TwoEnvIsolationRuleSpec import TwoEnvIsolationRuleSpec
from ntnx_microseg_py_client.models.microseg.v4.config.UdpPortRangeSpec import UdpPortRangeSpec
from ntnx_microseg_py_client.models.microseg.v4.config.UpdateAddressGroupApiResponse import UpdateAddressGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.UpdateDirectoryServerConfigApiResponse import UpdateDirectoryServerConfigApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.UpdateDsCategoryMappingApiResponse import UpdateDsCategoryMappingApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.UpdateEntityGroupApiResponse import UpdateEntityGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.UpdateNetworkSecurityPolicyApiResponse import UpdateNetworkSecurityPolicyApiResponse
from ntnx_microseg_py_client.models.microseg.v4.config.UpdateServiceGroupApiResponse import UpdateServiceGroupApiResponse
from ntnx_microseg_py_client.models.microseg.v4.error.AppMessage import AppMessage
from ntnx_microseg_py_client.models.microseg.v4.error.ErrorResponse import ErrorResponse
from ntnx_microseg_py_client.models.microseg.v4.error.SchemaValidationError import SchemaValidationError
from ntnx_microseg_py_client.models.microseg.v4.error.SchemaValidationErrorMessage import SchemaValidationErrorMessage
from ntnx_microseg_py_client.models.microseg.v4.operation.DryRunSpec import DryRunSpec
from ntnx_microseg_py_client.models.microseg.v4.operation.DryRunSummary import DryRunSummary
from ntnx_microseg_py_client.models.microseg.v4.operation.PolicyCount import PolicyCount
from ntnx_microseg_py_client.models.microseg.v4.operation.PolicyCountByType import PolicyCountByType
from ntnx_microseg_py_client.models.microseg.v4.operation.PolicyMetadata import PolicyMetadata
from ntnx_microseg_py_client.models.microseg.v4.operation.PolicyPreview import PolicyPreview
from ntnx_microseg_py_client.models.microseg.v4.operation.PolicyUpgradeSummary import PolicyUpgradeSummary
from ntnx_microseg_py_client.models.microseg.v4.operation.SecurityPolicyOptions import SecurityPolicyOptions
from ntnx_microseg_py_client.models.microseg.v4.operation.SecurityPolicyType import SecurityPolicyType as OperationSecurityPolicyType
from ntnx_microseg_py_client.models.microseg.v4.operation.SubnetUpgradeSummary import SubnetUpgradeSummary
from ntnx_microseg_py_client.models.microseg.v4.operation.TotalPolicyCount import TotalPolicyCount
from ntnx_microseg_py_client.models.microseg.v4.operation.UpgradeJob import UpgradeJob
from ntnx_microseg_py_client.models.microseg.v4.operation.UpgradeSpec import UpgradeSpec
from ntnx_microseg_py_client.models.prism.v4.config.TaskReference import TaskReference
from ntnx_microseg_py_client.api_response import ApiResponse



# coding: utf-8


"""
IGNORE:
    Nutanix Flow Management APIs

    Manage Network Security Policy configuration of Nutanix clusters.  # noqa: E501

    OpenAPI spec version: 4.2.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import pprint
import json
import ast
import re  # noqa: F401

import six
from pathlib import Path
from ntnx_microseg_py_client.models.common.v1.config.IPv4Address import IPv4Address  # noqa: F401,E501
from ntnx_microseg_py_client.models.common.v1.config.IPv6Address import IPv6Address  # noqa: F401,E501
from ntnx_microseg_py_client.models.common.v1.response.ApiLink import ApiLink  # noqa: F401,E501
from ntnx_microseg_py_client.models.common.v1.response.ExternalizableAbstractModel import ExternalizableAbstractModel  # noqa: F401,E501
from ntnx_microseg_py_client.models.microseg.v4.config.IPv4Range import IPv4Range  # noqa: F401,E501
from ntnx_microseg_py_client.models.microseg.v4.config.IPv6Range import IPv6Range  # noqa: F401,E501

class AddressGroup(ExternalizableAbstractModel):

    """

    :param name: (:attr:`name`) A short identifier for an Address Group.
    :type name: required
     max=64 chars
    
    :param description: (:attr:`description`) A user defined annotation for an Address Group.
    :type description: 
     max=1000 chars
    
    :param ipv4_addresses: (:attr:`ipv4_addresses`) List of CIDR blocks in the Address Group.
    :type ipv4_addresses: 
    
    
    :param ip_ranges: (:attr:`ip_ranges`) List of IP range containing start and end IP.
    :type ip_ranges: 
    
    
    :param ipv6_addresses: (:attr:`ipv6_addresses`) List of Ipv6 CIDR blocks in the Address Group.
    :type ipv6_addresses: 
    
    
    :param ipv6_ranges: (:attr:`ipv6_ranges`) List of IPv6 range containing start and end IP.
    :type ipv6_ranges: 
    
    
    :param policy_references: (:attr:`policy_references`) Reference to policy associated with Address Group.
    :type policy_references: 
    
    
    :param created_by: (:attr:`created_by`) The external identifier of the user who created the Address Group.
    :type created_by: 
    
    
    :param creation_time: (:attr:`creation_time`) The timestamp when the Address Group was created.
    :type creation_time: 
    
    
    :param last_update_time: (:attr:`last_update_time`) The timestamp when the Address Group was last updated.
    :type last_update_time: 
    
    

    """
    """
    IGNORE:
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    IGNORE
    """  # noqa: E501
    swagger_types = {
        'name': 'str',
        'description': 'str',
        'ipv4_addresses': 'list[common.v1.config.IPv4Address]',
        'ip_ranges': 'list[microseg.v4.config.IPv4Range]',
        'ipv6_addresses': 'list[common.v1.config.IPv6Address]',
        'ipv6_ranges': 'list[microseg.v4.config.IPv6Range]',
        'policy_references': 'list[str]',
        'created_by': 'str',
        'creation_time': 'datetime',
        'last_update_time': 'datetime',
        'ext_id': 'str',
        'links': 'list[common.v1.response.ApiLink]',
        'tenant_id': 'str',
        '_reserved': 'dict(str, object)',
        '_object_type': 'str',
        '_unknown_fields': 'dict(str, object)',
    }

    attribute_map = {
        'name': 'name',
        'description': 'description',
        'ipv4_addresses': 'ipv4Addresses',
        'ip_ranges': 'ipRanges',
        'ipv6_addresses': 'ipv6Addresses',
        'ipv6_ranges': 'ipv6Ranges',
        'policy_references': 'policyReferences',
        'created_by': 'createdBy',
        'creation_time': 'creationTime',
        'last_update_time': 'lastUpdateTime',
        'ext_id': 'extId',
        'links': 'links',
        'tenant_id': 'tenantId',
        '_reserved': '$reserved',
        '_object_type': '$objectType',
        '_unknown_fields': '$unknownFields',
    }



    def __init__(self, name=None, description=None, ipv4_addresses=None, ip_ranges=None, ipv6_addresses=None, ipv6_ranges=None, policy_references=None, created_by=None, creation_time=None, last_update_time=None, ext_id=None, links=None, tenant_id=None, *args, **kwargs):  # noqa: E501
        ExternalizableAbstractModel.__init__(self, ext_id, links, tenant_id, *args, **kwargs)
        self.__name = None
        self.__description = None
        self.__ipv4_addresses = None
        self.__ip_ranges = None
        self.__ipv6_addresses = None
        self.__ipv6_ranges = None
        self.__policy_references = None
        self.__created_by = None
        self.__creation_time = None
        self.__last_update_time = None
        self.discriminator = None
        self.__name = name
        if description is not None:
            self.__description = description
        if ipv4_addresses is not None:
            self.__ipv4_addresses = ipv4_addresses
        if ip_ranges is not None:
            self.__ip_ranges = ip_ranges
        if ipv6_addresses is not None:
            self.__ipv6_addresses = ipv6_addresses
        if ipv6_ranges is not None:
            self.__ipv6_ranges = ipv6_ranges
        if policy_references is not None:
            self.__policy_references = policy_references
        if created_by is not None:
            self.__created_by = created_by
        if creation_time is not None:
            self.__creation_time = creation_time
        if last_update_time is not None:
            self.__last_update_time = last_update_time

    def _initialize_object_type(self):
        return 'microseg.v4.config.AddressGroup'

    def _initialize_object_version(self):
        return 'v4.r2'


    @property
    def name(self):
        """
        A short identifier for an Address Group.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__name

    @name.setter
    def name(self, name):
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501
        if name is not None and len(name) > 64:
            raise ValueError("Invalid value for `name`, length must be less than or equal to `64`")  # noqa: E501
        if name is not None and not re.search('^\\s*[a-zA-Z0-9]([a-zA-Z0-9-_ \\\\.]*[a-zA-Z0-9])?\\s*$', name):  # noqa: E501
            raise ValueError(r"Invalid value for `name`, must be a follow pattern or equal to `/^\\s*[a-zA-Z0-9]([a-zA-Z0-9-_ \\\\.]*[a-zA-Z0-9])?\\s*$/`")  # noqa: E501

        self.__name = name

    @property
    def description(self):
        """
        A user defined annotation for an Address Group.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__description

    @description.setter
    def description(self, description):
        if description is not None and len(description) > 1000:
            raise ValueError("Invalid value for `description`, length must be less than or equal to `1000`")  # noqa: E501
        if description is not None and not re.search('^\\s*([a-zA-Z0-9 _:.()-]*)\\s*$', description):  # noqa: E501
            raise ValueError(r"Invalid value for `description`, must be a follow pattern or equal to `/^\\s*([a-zA-Z0-9 _:.()-]*)\\s*$/`")  # noqa: E501

        self.__description = description

    @property
    def ipv4_addresses(self):
        """
        List of CIDR blocks in the Address Group.

        :type:
             list[ :class:`~ntnx_microseg_py_client.models.common.v1.config.IPv4Address` ]
        """  # noqa: E501
        return self.__ipv4_addresses

    @ipv4_addresses.setter
    def ipv4_addresses(self, ipv4_addresses):

        self.__ipv4_addresses = ipv4_addresses

    @property
    def ip_ranges(self):
        """
        List of IP range containing start and end IP.

        :type:
             list[ :class:`~ntnx_microseg_py_client.models.microseg.v4.config.IPv4Range` ]
        """  # noqa: E501
        return self.__ip_ranges

    @ip_ranges.setter
    def ip_ranges(self, ip_ranges):

        self.__ip_ranges = ip_ranges

    @property
    def ipv6_addresses(self):
        """
        List of Ipv6 CIDR blocks in the Address Group.

        :type:
             list[ :class:`~ntnx_microseg_py_client.models.common.v1.config.IPv6Address` ]
        """  # noqa: E501
        return self.__ipv6_addresses

    @ipv6_addresses.setter
    def ipv6_addresses(self, ipv6_addresses):

        self.__ipv6_addresses = ipv6_addresses

    @property
    def ipv6_ranges(self):
        """
        List of IPv6 range containing start and end IP.

        :type:
             list[ :class:`~ntnx_microseg_py_client.models.microseg.v4.config.IPv6Range` ]
        """  # noqa: E501
        return self.__ipv6_ranges

    @ipv6_ranges.setter
    def ipv6_ranges(self, ipv6_ranges):

        self.__ipv6_ranges = ipv6_ranges

    @property
    def policy_references(self):
        """
        Reference to policy associated with Address Group.

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__policy_references

    @policy_references.setter
    def policy_references(self, policy_references):

        self.__policy_references = policy_references

    @property
    def created_by(self):
        """
        The external identifier of the user who created the Address Group.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__created_by

    @created_by.setter
    def created_by(self, created_by):
        if created_by is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', created_by):  # noqa: E501
            raise ValueError(r"Invalid value for `created_by`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__created_by = created_by

    @property
    def creation_time(self):
        """
        The timestamp when the Address Group was created.

        :type:
                :class:`~datetime`
        """  # noqa: E501
        return self.__creation_time

    @creation_time.setter
    def creation_time(self, creation_time):

        self.__creation_time = creation_time

    @property
    def last_update_time(self):
        """
        The timestamp when the Address Group was last updated.

        :type:
                :class:`~datetime`
        """  # noqa: E501
        return self.__last_update_time

    @last_update_time.setter
    def last_update_time(self, last_update_time):

        self.__last_update_time = last_update_time

    def _to_dict(self, sanitize):
        """Returns the model properties as a dict. Omits None properties based on the provided "sanitize" parameter.

        :param sanitize: A flag to omit None properties if set to True
        :type sanitize: bool
        """

        result = {}
        for attr, attr_type in six.iteritems(self.swagger_types):

            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x._to_dict(sanitize) if hasattr(x, "_to_dict") else x,
                    value
                ))
            elif hasattr(value, "_to_dict"):
                result[attr] = value._to_dict(sanitize)
            elif isinstance(value, Path):
                result[attr] = str(value)
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1]._to_dict(sanitize)) if hasattr(item[1], "_to_dict")
                    else (
                        (item[0], str(item[1])) if item[0] == 'path' and '$objectType' in value and value['$objectType'] == 'pathlib.Path'
                        else item
                    ),
                    six.iteritems(value)
                ))
            else:
                if sanitize:
                    if value is not None:
                        result[attr] = value
                else:
                    result[attr] = value
        if issubclass(AddressGroup, dict):
            for key, value in six.iteritems(self):
                result[key] = value

        # handle unknown fields
        result['_unknown_fields'] = {}
        unknown = self.get_unknown_fields()
        if sanitize:
            unknown = self._sanitize_for_serialization(unknown)

        return {**result, **unknown}


    def to_dict(self):
        """Returns the model properties as a dictionary"""
        return self._to_dict(False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self._to_dict(True))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AddressGroup):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other




# coding: utf-8


"""
IGNORE:
    Nutanix Flow Management APIs

    Manage Network Security Policy configuration of Nutanix clusters.  # noqa: E501

    OpenAPI spec version: 4.2.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import pprint
import json
import ast
import re  # noqa: F401

import six
from pathlib import Path
import ntnx_microseg_py_client.models
from ntnx_microseg_py_client.models.OneOfmicroseg.v4.config.NetworkSecurityPolicyRulespec import NetworkSecurityPolicyRulespec  # noqa: F401,E501
from ntnx_microseg_py_client.models.common.v1.response.ApiLink import ApiLink  # noqa: F401,E501
from ntnx_microseg_py_client.models.common.v1.response.ExternalizableAbstractModel import ExternalizableAbstractModel  # noqa: F401,E501
from ntnx_microseg_py_client.models.microseg.v4.config.RuleType import RuleType  # noqa: F401,E501

class NetworkSecurityPolicyRule(ExternalizableAbstractModel):

    """

    :param description: (:attr:`description`) A user defined annotation for a rule.
    :type description: 
     max=64 chars
    
    :param type: (:attr:`type`) 
    :type type: required
    
    
    :param spec: (:attr:`spec`) The type of the Flow Network Security policy rule spec.
    :type spec: required
    
    

    """
    """
    IGNORE:
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    IGNORE
    """  # noqa: E501
    swagger_types = {
        'description': 'str',
        'type': 'microseg.v4.config.RuleType',
        'spec': 'OneOfmicroseg.v4.config.NetworkSecurityPolicyRulespec',
        'ext_id': 'str',
        'links': 'list[common.v1.response.ApiLink]',
        'tenant_id': 'str',
        '_reserved': 'dict(str, object)',
        '_object_type': 'str',
        '_unknown_fields': 'dict(str, object)',
    }

    attribute_map = {
        'description': 'description',
        'type': 'type',
        'spec': 'spec',
        'ext_id': 'extId',
        'links': 'links',
        'tenant_id': 'tenantId',
        '_reserved': '$reserved',
        '_object_type': '$objectType',
        '_unknown_fields': '$unknownFields',
    }



    def __init__(self, description=None, type=None, spec=None, ext_id=None, links=None, tenant_id=None, *args, **kwargs):  # noqa: E501
        ExternalizableAbstractModel.__init__(self, ext_id, links, tenant_id, *args, **kwargs)
        self.__description = None
        self.__type = None
        self.__spec = None
        self.discriminator = None
        if description is not None:
            self.__description = description
        self.__type = type
        self.__spec = spec

    def _initialize_object_type(self):
        return 'microseg.v4.config.NetworkSecurityPolicyRule'

    def _initialize_object_version(self):
        return 'v4.r2'


    @property
    def description(self):
        """
        A user defined annotation for a rule.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__description

    @description.setter
    def description(self, description):
        if description is not None and len(description) > 64:
            raise ValueError("Invalid value for `description`, length must be less than or equal to `64`")  # noqa: E501

        self.__description = description

    @property
    def type(self):
        """
        

        :type:
            :class:`~ntnx_microseg_py_client.models.microseg.v4.config.RuleType`
        """  # noqa: E501
        return self.__type

    @type.setter
    def type(self, type):
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self.__type = type

    @property
    def spec(self):
        """
        The type of the Flow Network Security policy rule spec.

        :type:
                :class:`~ntnx_microseg_py_client.models.microseg.v4.config.TwoEnvIsolationRuleSpec` | 
                :class:`~ntnx_microseg_py_client.models.microseg.v4.config.ApplicationRuleSpec` | 
                :class:`~ntnx_microseg_py_client.models.microseg.v4.config.IntraEntityGroupRuleSpec` | 
                :class:`~ntnx_microseg_py_client.models.microseg.v4.config.MultiEnvIsolationRuleSpec`
        """  # noqa: E501
        return self.__spec

    @spec.setter
    def spec(self, spec):
        if spec is None:
            raise ValueError("Invalid value for `spec`, must not be `None`")  # noqa: E501

        self.__spec = spec

    def _to_dict(self, sanitize):
        """Returns the model properties as a dict. Omits None properties based on the provided "sanitize" parameter.

        :param sanitize: A flag to omit None properties if set to True
        :type sanitize: bool
        """

        result = {}
        for attr, attr_type in six.iteritems(self.swagger_types):

            value = getattr(self, attr)
            if attr_type.startswith('OneOf'):
                type = getattr(ntnx_microseg_py_client.models, attr_type.split('.')[-1])
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x._to_dict(sanitize) if hasattr(x, "_to_dict") else x,
                    value
                ))
            elif hasattr(value, "_to_dict"):
                result[attr] = value._to_dict(sanitize)
            elif isinstance(value, Path):
                result[attr] = str(value)
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1]._to_dict(sanitize)) if hasattr(item[1], "_to_dict")
                    else (
                        (item[0], str(item[1])) if item[0] == 'path' and '$objectType' in value and value['$objectType'] == 'pathlib.Path'
                        else item
                    ),
                    six.iteritems(value)
                ))
            else:
                if sanitize:
                    if value is not None:
                        result[attr] = value
                else:
                    result[attr] = value
        if issubclass(NetworkSecurityPolicyRule, dict):
            for key, value in six.iteritems(self):
                result[key] = value

        # handle unknown fields
        result['_unknown_fields'] = {}
        unknown = self.get_unknown_fields()
        if sanitize:
            unknown = self._sanitize_for_serialization(unknown)

        return {**result, **unknown}


    def to_dict(self):
        """Returns the model properties as a dictionary"""
        return self._to_dict(False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self._to_dict(True))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NetworkSecurityPolicyRule):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other




# coding: utf-8


"""
IGNORE:
    Nutanix Flow Management APIs

    Manage Network Security Policy configuration of Nutanix clusters.  # noqa: E501

    OpenAPI spec version: 4.2.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
IGNORE
"""
import pprint
import json
import ast
import re  # noqa: F401

import six
from pathlib import Path
from ntnx_microseg_py_client.models.common.v1.config.IPv4Address import IPv4Address  # noqa: F401,E501
from ntnx_microseg_py_client.models.common.v1.config.IPv6Address import IPv6Address  # noqa: F401,E501
from ntnx_microseg_py_client.models.microseg.v4.config.AllowType import AllowType  # noqa: F401,E501
from ntnx_microseg_py_client.models.microseg.v4.config.CategoryAssociatedEntityType import CategoryAssociatedEntityType  # noqa: F401,E501
from ntnx_microseg_py_client.models.microseg.v4.config.IcmpTypeCodeSpec import IcmpTypeCodeSpec  # noqa: F401,E501
from ntnx_microseg_py_client.models.microseg.v4.config.IcmpV6TypeCodeSpec import IcmpV6TypeCodeSpec  # noqa: F401,E501
from ntnx_microseg_py_client.models.microseg.v4.config.TcpPortRangeSpec import TcpPortRangeSpec  # noqa: F401,E501
from ntnx_microseg_py_client.models.microseg.v4.config.UdpPortRangeSpec import UdpPortRangeSpec  # noqa: F401,E501

class ApplicationRuleSpec(object):

    """A rule for specifying allowed traffic for an application.

    :param secured_group_category_associated_entity_type: (:attr:`secured_group_category_associated_entity_type`) 
    :type secured_group_category_associated_entity_type: 
    
    
    :param secured_group_category_references: (:attr:`secured_group_category_references`) A set of network endpoints which is protected by a Network Security Policy and defined as a list of categories.
    :type secured_group_category_references: 
    
    
    :param secured_group_entity_group_reference: (:attr:`secured_group_entity_group_reference`) External identifier of the entity group belonging to the secured group.
    :type secured_group_entity_group_reference: 
    
    
    :param src_allow_spec: (:attr:`src_allow_spec`) 
    :type src_allow_spec: 
    
    
    :param dest_allow_spec: (:attr:`dest_allow_spec`) 
    :type dest_allow_spec: 
    
    
    :param src_category_associated_entity_type: (:attr:`src_category_associated_entity_type`) 
    :type src_category_associated_entity_type: 
    
    
    :param src_category_references: (:attr:`src_category_references`) List of categories that define a set of network endpoints as inbound.
    :type src_category_references: 
    
    
    :param src_entity_group_reference: (:attr:`src_entity_group_reference`) External identifier of the entity group belonging to the source group.
    :type src_entity_group_reference: 
    
    
    :param dest_category_associated_entity_type: (:attr:`dest_category_associated_entity_type`) 
    :type dest_category_associated_entity_type: 
    
    
    :param dest_category_references: (:attr:`dest_category_references`) List of categories that define a set of network endpoints as outbound.
    :type dest_category_references: 
    
    
    :param dest_entity_group_reference: (:attr:`dest_entity_group_reference`) External identifier of the entity group belonging to the destination group.
    :type dest_entity_group_reference: 
    
    
    :param src_subnet: (:attr:`src_subnet`) 
    :type src_subnet: 
    
    
    :param src_ipv6_subnet: (:attr:`src_ipv6_subnet`) 
    :type src_ipv6_subnet: 
    
    
    :param dest_subnet: (:attr:`dest_subnet`) 
    :type dest_subnet: 
    
    
    :param dest_ipv6_subnet: (:attr:`dest_ipv6_subnet`) 
    :type dest_ipv6_subnet: 
    
    
    :param src_address_group_references: (:attr:`src_address_group_references`) A list of address group references.
    :type src_address_group_references: 
    
    
    :param dest_address_group_references: (:attr:`dest_address_group_references`) A list of address group references.
    :type dest_address_group_references: 
    
    
    :param service_group_references: (:attr:`service_group_references`) A list of service group references.
    :type service_group_references: 
    
    
    :param is_all_protocol_allowed: (:attr:`is_all_protocol_allowed`) Denotes whether the rule allows traffic for all protocols. If set to true, the rule allows traffic for all protocols. If set to false or not specified, specifying at least one protocol service or service group is mandatory.
    :type is_all_protocol_allowed: 
    
    
    :param tcp_services: (:attr:`tcp_services`) 
    :type tcp_services: 
    
    
    :param udp_services: (:attr:`udp_services`) 
    :type udp_services: 
    
    
    :param icmp_services: (:attr:`icmp_services`) ICMP Type Code List.
    :type icmp_services: 
    
    
    :param icmp_v6_services: (:attr:`icmp_v6_services`) ICMPv6 Type Code List.
    :type icmp_v6_services: 
    
    
    :param network_function_chain_reference: (:attr:`network_function_chain_reference`) A reference to the network function chain in the rule.
    :type network_function_chain_reference: 
    
    
    :param network_function_reference: (:attr:`network_function_reference`) A reference to the network function in the rule.
    :type network_function_reference: 
    
    

    """
    """
    IGNORE:
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    IGNORE
    """  # noqa: E501
    swagger_types = {
        'secured_group_category_associated_entity_type': 'microseg.v4.config.CategoryAssociatedEntityType',
        'secured_group_category_references': 'list[str]',
        'secured_group_entity_group_reference': 'str',
        'src_allow_spec': 'microseg.v4.config.AllowType',
        'dest_allow_spec': 'microseg.v4.config.AllowType',
        'src_category_associated_entity_type': 'microseg.v4.config.CategoryAssociatedEntityType',
        'src_category_references': 'list[str]',
        'src_entity_group_reference': 'str',
        'dest_category_associated_entity_type': 'microseg.v4.config.CategoryAssociatedEntityType',
        'dest_category_references': 'list[str]',
        'dest_entity_group_reference': 'str',
        'src_subnet': 'common.v1.config.IPv4Address',
        'src_ipv6_subnet': 'common.v1.config.IPv6Address',
        'dest_subnet': 'common.v1.config.IPv4Address',
        'dest_ipv6_subnet': 'common.v1.config.IPv6Address',
        'src_address_group_references': 'list[str]',
        'dest_address_group_references': 'list[str]',
        'service_group_references': 'list[str]',
        'is_all_protocol_allowed': 'bool',
        'tcp_services': 'list[microseg.v4.config.TcpPortRangeSpec]',
        'udp_services': 'list[microseg.v4.config.UdpPortRangeSpec]',
        'icmp_services': 'list[microseg.v4.config.IcmpTypeCodeSpec]',
        'icmp_v6_services': 'list[microseg.v4.config.IcmpV6TypeCodeSpec]',
        'network_function_chain_reference': 'str',
        'network_function_reference': 'str',
        '_reserved': 'dict(str, object)',
        '_object_type': 'str',
        '_unknown_fields': 'dict(str, object)',
    }

    attribute_map = {
        'secured_group_category_associated_entity_type': 'securedGroupCategoryAssociatedEntityType',
        'secured_group_category_references': 'securedGroupCategoryReferences',
        'secured_group_entity_group_reference': 'securedGroupEntityGroupReference',
        'src_allow_spec': 'srcAllowSpec',
        'dest_allow_spec': 'destAllowSpec',
        'src_category_associated_entity_type': 'srcCategoryAssociatedEntityType',
        'src_category_references': 'srcCategoryReferences',
        'src_entity_group_reference': 'srcEntityGroupReference',
        'dest_category_associated_entity_type': 'destCategoryAssociatedEntityType',
        'dest_category_references': 'destCategoryReferences',
        'dest_entity_group_reference': 'destEntityGroupReference',
        'src_subnet': 'srcSubnet',
        'src_ipv6_subnet': 'srcIpv6Subnet',
        'dest_subnet': 'destSubnet',
        'dest_ipv6_subnet': 'destIpv6Subnet',
        'src_address_group_references': 'srcAddressGroupReferences',
        'dest_address_group_references': 'destAddressGroupReferences',
        'service_group_references': 'serviceGroupReferences',
        'is_all_protocol_allowed': 'isAllProtocolAllowed',
        'tcp_services': 'tcpServices',
        'udp_services': 'udpServices',
        'icmp_services': 'icmpServices',
        'icmp_v6_services': 'icmpV6Services',
        'network_function_chain_reference': 'networkFunctionChainReference',
        'network_function_reference': 'networkFunctionReference',
        '_reserved': '$reserved',
        '_object_type': '$objectType',
        '_unknown_fields': '$unknownFields',
    }



    def __init__(self, secured_group_category_associated_entity_type=None, secured_group_category_references=None, secured_group_entity_group_reference=None, src_allow_spec=None, dest_allow_spec=None, src_category_associated_entity_type=None, src_category_references=None, src_entity_group_reference=None, dest_category_associated_entity_type=None, dest_category_references=None, dest_entity_group_reference=None, src_subnet=None, src_ipv6_subnet=None, dest_subnet=None, dest_ipv6_subnet=None, src_address_group_references=None, dest_address_group_references=None, service_group_references=None, is_all_protocol_allowed=None, tcp_services=None, udp_services=None, icmp_services=None, icmp_v6_services=None, network_function_chain_reference=None, network_function_reference=None, *args, **kwargs):  # noqa: E501
        self.__secured_group_category_associated_entity_type = None
        self.__secured_group_category_references = None
        self.__secured_group_entity_group_reference = None
        self.__src_allow_spec = None
        self.__dest_allow_spec = None
        self.__src_category_associated_entity_type = None
        self.__src_category_references = None
        self.__src_entity_group_reference = None
        self.__dest_category_associated_entity_type = None
        self.__dest_category_references = None
        self.__dest_entity_group_reference = None
        self.__src_subnet = None
        self.__src_ipv6_subnet = None
        self.__dest_subnet = None
        self.__dest_ipv6_subnet = None
        self.__src_address_group_references = None
        self.__dest_address_group_references = None
        self.__service_group_references = None
        self.__is_all_protocol_allowed = None
        self.__tcp_services = None
        self.__udp_services = None
        self.__icmp_services = None
        self.__icmp_v6_services = None
        self.__network_function_chain_reference = None
        self.__network_function_reference = None
        self.discriminator = None
        if secured_group_category_associated_entity_type is not None:
            self.__secured_group_category_associated_entity_type = secured_group_category_associated_entity_type
        if secured_group_category_references is not None:
            self.__secured_group_category_references = secured_group_category_references
        if secured_group_entity_group_reference is not None:
            self.__secured_group_entity_group_reference = secured_group_entity_group_reference
        if src_allow_spec is not None:
            self.__src_allow_spec = src_allow_spec
        if dest_allow_spec is not None:
            self.__dest_allow_spec = dest_allow_spec
        if src_category_associated_entity_type is not None:
            self.__src_category_associated_entity_type = src_category_associated_entity_type
        if src_category_references is not None:
            self.__src_category_references = src_category_references
        if src_entity_group_reference is not None:
            self.__src_entity_group_reference = src_entity_group_reference
        if dest_category_associated_entity_type is not None:
            self.__dest_category_associated_entity_type = dest_category_associated_entity_type
        if dest_category_references is not None:
            self.__dest_category_references = dest_category_references
        if dest_entity_group_reference is not None:
            self.__dest_entity_group_reference = dest_entity_group_reference
        if src_subnet is not None:
            self.__src_subnet = src_subnet
        if src_ipv6_subnet is not None:
            self.__src_ipv6_subnet = src_ipv6_subnet
        if dest_subnet is not None:
            self.__dest_subnet = dest_subnet
        if dest_ipv6_subnet is not None:
            self.__dest_ipv6_subnet = dest_ipv6_subnet
        if src_address_group_references is not None:
            self.__src_address_group_references = src_address_group_references
        if dest_address_group_references is not None:
            self.__dest_address_group_references = dest_address_group_references
        if service_group_references is not None:
            self.__service_group_references = service_group_references
        if is_all_protocol_allowed is not None:
            self.__is_all_protocol_allowed = is_all_protocol_allowed
        if tcp_services is not None:
            self.__tcp_services = tcp_services
        if udp_services is not None:
            self.__udp_services = udp_services
        if icmp_services is not None:
            self.__icmp_services = icmp_services
        if icmp_v6_services is not None:
            self.__icmp_v6_services = icmp_v6_services
        if network_function_chain_reference is not None:
            self.__network_function_chain_reference = network_function_chain_reference
        if network_function_reference is not None:
            self.__network_function_reference = network_function_reference
        # populate hidden vars if not empty
        self._populate_hidden_vars(kwargs)

    def _initialize_object_type(self):
        return 'microseg.v4.config.ApplicationRuleSpec'

    def _initialize_object_version(self):
        return 'v4.r2'

    def _populate_hidden_vars(self, kwargs):
        if "_reserved" in kwargs and kwargs["_reserved"] is not None:
            self.__dollar_reserved = kwargs["_reserved"]
        elif hasattr(self, "attribute_map") and "_reserved" in self.attribute_map and self.attribute_map["_reserved"] in kwargs and kwargs[self.attribute_map["_reserved"]] is not None:
            self.__dollar_reserved = kwargs[self.attribute_map["_reserved"]]
        else :
            self.__dollar_reserved = {"$fv": self._initialize_object_version()}
        if "_unknown_fields" in kwargs and kwargs["_unknown_fields"] is not None:
            self.__dollar_unknown_fields = kwargs["_unknown_fields"]
        elif hasattr(self, "attribute_map") and "_unknown_fields" in self.attribute_map and self.attribute_map["_unknown_fields"] in kwargs and kwargs[self.attribute_map["_unknown_fields"]] is not None:
            self.__dollar_unknown_fields = kwargs[self.attribute_map["_unknown_fields"]]
        else :
            self.__dollar_unknown_fields = {}
        if "_object_type" in kwargs and kwargs["_object_type"] is not None:
            self.__dollar_object_type = kwargs["_object_type"]
        elif hasattr(self, "attribute_map") and "_object_type" in self.attribute_map and self.attribute_map["_object_type"] in kwargs and kwargs[self.attribute_map["_object_type"]] is not None:
            self.__dollar_object_type = kwargs[self.attribute_map["_object_type"]]
        else:
            self.__dollar_object_type = self._initialize_object_type()

    def get_object_type(self):
        return self.__dollar_object_type

    def get_reserved(self):
        return self.__dollar_reserved

    def get_unknown_fields(self):
        return self.__dollar_unknown_fields

    @property
    def secured_group_category_associated_entity_type(self):
        """
        

        :type:
            :class:`~ntnx_microseg_py_client.models.microseg.v4.config.CategoryAssociatedEntityType`
        """  # noqa: E501
        return self.__secured_group_category_associated_entity_type

    @secured_group_category_associated_entity_type.setter
    def secured_group_category_associated_entity_type(self, secured_group_category_associated_entity_type):

        self.__secured_group_category_associated_entity_type = secured_group_category_associated_entity_type

    @property
    def secured_group_category_references(self):
        """
        A set of network endpoints which is protected by a Network Security Policy and defined as a list of categories.

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__secured_group_category_references

    @secured_group_category_references.setter
    def secured_group_category_references(self, secured_group_category_references):

        self.__secured_group_category_references = secured_group_category_references

    @property
    def secured_group_entity_group_reference(self):
        """
        External identifier of the entity group belonging to the secured group.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__secured_group_entity_group_reference

    @secured_group_entity_group_reference.setter
    def secured_group_entity_group_reference(self, secured_group_entity_group_reference):
        if secured_group_entity_group_reference is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', secured_group_entity_group_reference):  # noqa: E501
            raise ValueError(r"Invalid value for `secured_group_entity_group_reference`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__secured_group_entity_group_reference = secured_group_entity_group_reference

    @property
    def src_allow_spec(self):
        """
        

        :type:
            :class:`~ntnx_microseg_py_client.models.microseg.v4.config.AllowType`
        """  # noqa: E501
        return self.__src_allow_spec

    @src_allow_spec.setter
    def src_allow_spec(self, src_allow_spec):

        self.__src_allow_spec = src_allow_spec

    @property
    def dest_allow_spec(self):
        """
        

        :type:
            :class:`~ntnx_microseg_py_client.models.microseg.v4.config.AllowType`
        """  # noqa: E501
        return self.__dest_allow_spec

    @dest_allow_spec.setter
    def dest_allow_spec(self, dest_allow_spec):

        self.__dest_allow_spec = dest_allow_spec

    @property
    def src_category_associated_entity_type(self):
        """
        

        :type:
            :class:`~ntnx_microseg_py_client.models.microseg.v4.config.CategoryAssociatedEntityType`
        """  # noqa: E501
        return self.__src_category_associated_entity_type

    @src_category_associated_entity_type.setter
    def src_category_associated_entity_type(self, src_category_associated_entity_type):

        self.__src_category_associated_entity_type = src_category_associated_entity_type

    @property
    def src_category_references(self):
        """
        List of categories that define a set of network endpoints as inbound.

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__src_category_references

    @src_category_references.setter
    def src_category_references(self, src_category_references):

        self.__src_category_references = src_category_references

    @property
    def src_entity_group_reference(self):
        """
        External identifier of the entity group belonging to the source group.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__src_entity_group_reference

    @src_entity_group_reference.setter
    def src_entity_group_reference(self, src_entity_group_reference):
        if src_entity_group_reference is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', src_entity_group_reference):  # noqa: E501
            raise ValueError(r"Invalid value for `src_entity_group_reference`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__src_entity_group_reference = src_entity_group_reference

    @property
    def dest_category_associated_entity_type(self):
        """
        

        :type:
            :class:`~ntnx_microseg_py_client.models.microseg.v4.config.CategoryAssociatedEntityType`
        """  # noqa: E501
        return self.__dest_category_associated_entity_type

    @dest_category_associated_entity_type.setter
    def dest_category_associated_entity_type(self, dest_category_associated_entity_type):

        self.__dest_category_associated_entity_type = dest_category_associated_entity_type

    @property
    def dest_category_references(self):
        """
        List of categories that define a set of network endpoints as outbound.

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__dest_category_references

    @dest_category_references.setter
    def dest_category_references(self, dest_category_references):

        self.__dest_category_references = dest_category_references

    @property
    def dest_entity_group_reference(self):
        """
        External identifier of the entity group belonging to the destination group.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__dest_entity_group_reference

    @dest_entity_group_reference.setter
    def dest_entity_group_reference(self, dest_entity_group_reference):
        if dest_entity_group_reference is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', dest_entity_group_reference):  # noqa: E501
            raise ValueError(r"Invalid value for `dest_entity_group_reference`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__dest_entity_group_reference = dest_entity_group_reference

    @property
    def src_subnet(self):
        """
        

        :type:
            :class:`~ntnx_microseg_py_client.models.common.v1.config.IPv4Address`
        """  # noqa: E501
        return self.__src_subnet

    @src_subnet.setter
    def src_subnet(self, src_subnet):

        self.__src_subnet = src_subnet

    @property
    def src_ipv6_subnet(self):
        """
        

        :type:
            :class:`~ntnx_microseg_py_client.models.common.v1.config.IPv6Address`
        """  # noqa: E501
        return self.__src_ipv6_subnet

    @src_ipv6_subnet.setter
    def src_ipv6_subnet(self, src_ipv6_subnet):

        self.__src_ipv6_subnet = src_ipv6_subnet

    @property
    def dest_subnet(self):
        """
        

        :type:
            :class:`~ntnx_microseg_py_client.models.common.v1.config.IPv4Address`
        """  # noqa: E501
        return self.__dest_subnet

    @dest_subnet.setter
    def dest_subnet(self, dest_subnet):

        self.__dest_subnet = dest_subnet

    @property
    def dest_ipv6_subnet(self):
        """
        

        :type:
            :class:`~ntnx_microseg_py_client.models.common.v1.config.IPv6Address`
        """  # noqa: E501
        return self.__dest_ipv6_subnet

    @dest_ipv6_subnet.setter
    def dest_ipv6_subnet(self, dest_ipv6_subnet):

        self.__dest_ipv6_subnet = dest_ipv6_subnet

    @property
    def src_address_group_references(self):
        """
        A list of address group references.

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__src_address_group_references

    @src_address_group_references.setter
    def src_address_group_references(self, src_address_group_references):

        self.__src_address_group_references = src_address_group_references

    @property
    def dest_address_group_references(self):
        """
        A list of address group references.

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__dest_address_group_references

    @dest_address_group_references.setter
    def dest_address_group_references(self, dest_address_group_references):

        self.__dest_address_group_references = dest_address_group_references

    @property
    def service_group_references(self):
        """
        A list of service group references.

        :type:
            list[ :class:`~str` ]
        """  # noqa: E501
        return self.__service_group_references

    @service_group_references.setter
    def service_group_references(self, service_group_references):

        self.__service_group_references = service_group_references

    @property
    def is_all_protocol_allowed(self):
        """
        Denotes whether the rule allows traffic for all protocols. If set to true, the rule allows traffic for all protocols. If set to false or not specified, specifying at least one protocol service or service group is mandatory.

        :type:
                :class:`~bool`
        """  # noqa: E501
        return self.__is_all_protocol_allowed

    @is_all_protocol_allowed.setter
    def is_all_protocol_allowed(self, is_all_protocol_allowed):

        self.__is_all_protocol_allowed = is_all_protocol_allowed

    @property
    def tcp_services(self):
        """
        

        :type:
             list[ :class:`~ntnx_microseg_py_client.models.microseg.v4.config.TcpPortRangeSpec` ]
        """  # noqa: E501
        return self.__tcp_services

    @tcp_services.setter
    def tcp_services(self, tcp_services):

        self.__tcp_services = tcp_services

    @property
    def udp_services(self):
        """
        

        :type:
             list[ :class:`~ntnx_microseg_py_client.models.microseg.v4.config.UdpPortRangeSpec` ]
        """  # noqa: E501
        return self.__udp_services

    @udp_services.setter
    def udp_services(self, udp_services):

        self.__udp_services = udp_services

    @property
    def icmp_services(self):
        """
        ICMP Type Code List.

        :type:
             list[ :class:`~ntnx_microseg_py_client.models.microseg.v4.config.IcmpTypeCodeSpec` ]
        """  # noqa: E501
        return self.__icmp_services

    @icmp_services.setter
    def icmp_services(self, icmp_services):

        self.__icmp_services = icmp_services

    @property
    def icmp_v6_services(self):
        """
        ICMPv6 Type Code List.

        :type:
             list[ :class:`~ntnx_microseg_py_client.models.microseg.v4.config.IcmpV6TypeCodeSpec` ]
        """  # noqa: E501
        return self.__icmp_v6_services

    @icmp_v6_services.setter
    def icmp_v6_services(self, icmp_v6_services):

        self.__icmp_v6_services = icmp_v6_services

    @property
    def network_function_chain_reference(self):
        """
        A reference to the network function chain in the rule.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__network_function_chain_reference

    @network_function_chain_reference.setter
    def network_function_chain_reference(self, network_function_chain_reference):
        if network_function_chain_reference is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', network_function_chain_reference):  # noqa: E501
            raise ValueError(r"Invalid value for `network_function_chain_reference`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__network_function_chain_reference = network_function_chain_reference

    @property
    def network_function_reference(self):
        """
        A reference to the network function in the rule.

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__network_function_reference

    @network_function_reference.setter
    def network_function_reference(self, network_function_reference):
        if network_function_reference is not None and not re.search('^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$', network_function_reference):  # noqa: E501
            raise ValueError(r"Invalid value for `network_function_reference`, must be a follow pattern or equal to `/^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$/`")  # noqa: E501

        self.__network_function_reference = network_function_reference

    @property
    def _reserved(self):
        """
        

        :type:
            dict(str, :class:`~object`)
        """  # noqa: E501
        return self.__dollar_reserved

    @property
    def _object_type(self):
        """
        

        :type:
                :class:`~str`
        """  # noqa: E501
        return self.__dollar_object_type

    @property
    def _unknown_fields(self):
        """
        

        :type:
            dict(str, :class:`~object`)
        """  # noqa: E501
        return self.__dollar_unknown_fields

    def _to_dict(self, sanitize):
        """Returns the model properties as a dict. Omits None properties based on the provided "sanitize" parameter.

        :param sanitize: A flag to omit None properties if set to True
        :type sanitize: bool
        """

        result = {}
        for attr, attr_type in six.iteritems(self.swagger_types):

            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x._to_dict(sanitize) if hasattr(x, "_to_dict") else x,
                    value
                ))
            elif hasattr(value, "_to_dict"):
                result[attr] = value._to_dict(sanitize)
            elif isinstance(value, Path):
                result[attr] = str(value)
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1]._to_dict(sanitize)) if hasattr(item[1], "_to_dict")
                    else (
                        (item[0], str(item[1])) if item[0] == 'path' and '$objectType' in value and value['$objectType'] == 'pathlib.Path'
                        else item
                    ),
                    six.iteritems(value)
                ))
            else:
                if sanitize:
                    if value is not None:
                        result[attr] = value
                else:
                    result[attr] = value
        if issubclass(ApplicationRuleSpec, dict):
            for key, value in six.iteritems(self):
                result[key] = value

        # handle unknown fields
        result['_unknown_fields'] = {}
        unknown = self.get_unknown_fields()
        if sanitize:
            unknown = self._sanitize_for_serialization(unknown)

        return {**result, **unknown}

    def _sanitize_for_serialization(self, value):
        """Recursively sanitize the value for serialization.

        :param value: The value to sanitize
        :type value: any
        :return: Sanitized value
        :rtype: any
        """
        if isinstance(value, (list, tuple)):
            return list(map(self._sanitize_for_serialization, [v for v in value if v is not None]))
        elif isinstance(value, dict):
            return {k: self._sanitize_for_serialization(v) for k, v in six.iteritems(value) if v is not None}
        elif isinstance(value, (str, int, float, bool)):
            return value
        elif isinstance(value, Path):
            return str(value)
        else:
            return value.to_dict() if hasattr(value, 'to_dict') else str(value)

    def to_dict(self):
        """Returns the model properties as a dictionary"""
        return self._to_dict(False)

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self._to_dict(True))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ApplicationRuleSpec):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other

